@@ -29,6 +29,7 @@ from ..base.model import TaggingModel
 from ..base.model_data import TaggingSample
 from ..base.model_dict import TagsDict, AbstractTagger
 from ..base.tokenizer import BertTokenizer, AbstractTokenizer
+from ..base.common import Metric
 
 
 class TaggingModelSettings(ModelSettings):
@@ -265,10 +266,9 @@ class Tagging(AbstractModelApp):
         :return: bool, average loss
         """
         loss_score = 0
-        count_loss = 0
-
-        correct_tags_count = 0
-        tags_count = 0
+        count_batch = 0
+        recall_score = 0.0
+        precision_score = 0.0
 
         data_loader = None
         if data_type == ModelDataType.VALID:
@@ -294,21 +294,26 @@ class Tagging(AbstractModelApp):
                 x = x.cuda()
                 y = y.cuda()
             y_ = self.model(x)
-            y_ = torch.transpose(y_, dim0=2, dim1=1)
-            loss_score = loss_score + float(self.loss_func(y_, y))
-            count_loss = count_loss + 1
+            loss_score = loss_score + float(self.loss_func(torch.transpose(y_, dim0=2, dim1=1), y))
+            count_batch = count_batch + 1
 
-            #
-            correct_tags_count = correct_tags_count + int(torch.sum(torch.eq(y, torch.max(y_, dim=1)[1])))
-            tags_count = tags_count + y.shape[0] * y.shape[1]
+            # 计算batch的 RPF 0.5 阈值
+            rpf = Metric.positional_weighted_rpf(y, scores=torch.exp(y_)[:,:,1], point_count=1)
+            recall_score = recall_score + float(torch.mean(rpf[0, :, 1]))
+            precision_score = precision_score + float(torch.mean(rpf[0, :, 2]))
+
+
+            # correct_tags_count = correct_tags_count + int(torch.sum(torch.eq(y, torch.max(y_, dim=1)[1])))
+            # tags_count = tags_count + y.shape[0] * y.shape[1]
 
         # 平均loss
-        ave_loss = (1.0 * loss_score) / (count_loss + Const.MIN_POSITIVE_NUMBER)
-        tag_acc = (1.0 * correct_tags_count) / (tags_count + Const.MIN_POSITIVE_NUMBER)
+        ave_loss = (1.0 * loss_score) / (count_batch + Const.MIN_POSITIVE_NUMBER)
+        ave_precision_score = (1.0 * precision_score) / (count_batch + Const.MIN_POSITIVE_NUMBER)
+        ave_recall_score = (1.0 * recall_score) / (count_batch + Const.MIN_POSITIVE_NUMBER)
 
         logging.info(
-            "Valid data Dataset %s TaggingCorrectCount %d TaggingCount %d TaggingAcc=%s" % (
-                data_type, correct_tags_count, tags_count, str(tag_acc)
+            "Valid data Dataset %s AverageLoss=%f AveragePrecision=%f AverageRecall=%f" % (
+                data_type, ave_loss, ave_precision_score, ave_recall_score
             )
         )
         return True, ave_loss
